import 'package:flutter/widgets.dart';
import 'package:flutter_test/flutter_test.dart';

import 'robot.dart';

extension RobotExpectation on Robot {
  void expectShown() => expect(this, findsOneWidget);
  void expectHidden() => expect(this, findsNothing);
}

extension RobotPositionExpectation on Robot {
  void expectPositionedAtTop({bool usingSafeArea = false}) {
    final widgetTop = tester.getTopLeft(this).dy;
    final screenTop = usingSafeArea ? tester.getTopLeft(find.byType(SafeArea)).dy : 0;
    expect(widgetTop, screenTop);
  }

  void expectPositionedAtBottom({bool usingSafeArea = false}) {
    final widgetBottom = tester.getBottomRight(this).dy;
    final screenBottom = usingSafeArea
        ? tester.getBottomRight(find.byType(SafeArea)).dy
        : tester.view.physicalSize.height / tester.view.devicePixelRatio;
    expect(widgetBottom, screenBottom);
  }

  void expectIsVerticallyAdjacentTo(Finder other) {
    final widgetBottom = tester.getBottomLeft(this).dy;
    final otherTop = tester.getTopLeft(other).dy;
    expect(widgetBottom, otherTop);
  }

  void expectIsHorizontallyAdjacentTo(Finder other) {
    final widgetRight = tester.getTopRight(this).dx;
    final otherLeft = tester.getTopLeft(other).dx;
    expect(widgetRight, otherLeft);
  }

  void expectIsVerticallyCentered() {
    final widgetCenter = tester.getCenter(this).dy;
    final screenCenter = tester.view.physicalSize.height / tester.view.devicePixelRatio / 2;
    expect(widgetCenter, closeTo(screenCenter, 0.1));
  }

  void expectIsHorizontallyCentered() {
    final widgetCenter = tester.getCenter(this).dx;
    final screenCenter = tester.view.physicalSize.width / tester.view.devicePixelRatio / 2;
    expect(widgetCenter, closeTo(screenCenter, 0.1));
  }

  void expectIsCentered() {
    expectIsVerticallyCentered();
    expectIsHorizontallyCentered();
  }

  void expectIsCenteredIn(Finder parent) {
    final widgetCenter = tester.getCenter(this);
    final parentCenter = tester.getCenter(parent);
    expect(widgetCenter.dx, closeTo(parentCenter.dx, 0.1));
    expect(widgetCenter.dy, closeTo(parentCenter.dy, 0.1));
  }

  void expectSpacedVerticallyTo(Finder other, {required double space}) {
    final widgetBottom = tester.getBottomLeft(this).dy;
    final otherTop = tester.getTopLeft(other).dy;
    expect(widgetBottom + space, otherTop);
  }

  void expectSpacedHorizontallyTo(
    Finder other, {
    required double space,
  }) {
    final widgetRight = tester.getTopRight(this).dx;
    final otherLeft = tester.getTopLeft(other).dx;
    expect(widgetRight + space, otherLeft);
  }
}
